@@ -32,18 +32,28 @@ use Test::More;
 require Exporter;
 use base qw/Exporter/;
 use Carp;
+use IO::Pipe;
+use File::Temp qw/tempdir/;
 
 our %disable_validations;
 our @EXPORT = qw/fake_config config_create_and_parse disable_validations no_disable_validations warning_fatal
-capture_stdout capture_stderr assert_raises_exception ordered_test test_fast_ok fast_ok/;
+capture_stdout capture_stderr assert_raises_exception ordered_test test_fast_ok fast_ok with_fork can_work_with_non_utf8_files get_temp_dir/;
 
 use Test::Deep; # should be last line, after EXPORT stuff, otherwise versions ^(0\.089|0\.09[0-9].*) do something nastly with exports
 
+use constant ALARM_FOR_FORK_TESTS => 30;
+
 sub warning_fatal
 {
 	$SIG{__WARN__} = sub {confess "Termination after a warning: $_[0]"};
 }
 
+sub get_temp_dir
+{
+	$SIG{INT} = sub { exit(1); }; # Global signal, for cleaning temporary files
+	tempdir("__AppMtAws_t_${$}_XXXXXXXX", TMPDIR => 1, CLEANUP => 1); # pid needed cause child processes re-use random number generators
+}
+
 sub fake_config(@)
 {
 	my ($cb, %data) = (pop @_, @_);
@@ -176,5 +186,52 @@ sub test_fast_ok
 	}
 }
 
+sub with_fork(&&)
+{
+	my ($parent_cb, $child_cb) = @_;
+	my $fromchild = new IO::Pipe;
+	my $tochild = new IO::Pipe;
+
+	if (my $pid = fork()) {
+		my $child_exited = 0;
+		$fromchild->reader();
+		$fromchild->autoflush(1);
+		$fromchild->blocking(1);
+		binmode $fromchild;
+
+		$tochild->writer();
+		$tochild->autoflush(1);
+		$tochild->blocking(1);
+		binmode $tochild;
+
+		alarm ALARM_FOR_FORK_TESTS; # protect from hang in case our test fail
+		$parent_cb->($fromchild, $tochild);
+		alarm 0;
+
+		while(wait() != -1 ){};
+	} else {
+		$fromchild->writer();
+		$fromchild->autoflush(1);
+		$fromchild->blocking(1);
+		binmode $fromchild;
+
+		$tochild->reader();
+		$tochild->autoflush(1);
+		$tochild->blocking(1);
+		binmode $tochild;
+
+		alarm ALARM_FOR_FORK_TESTS; # protect from hang in case our test fail
+		$child_cb->($tochild, $fromchild);
+		alarm 0;
+
+		exit(0);
+	}
+}
+
+
+sub can_work_with_non_utf8_files
+{
+	$^O =~ /^(linux|.*bsd|solaris)$/i;
+}
 
 1;
